@@ -2,14 +2,16 @@
 
 use crate::language::InternSymbol;
 use crate::parser::lvalue::DeclType;
+use crate::parser::stmt::Label;
 use crate::debug::symbol::DebugSymbol;
+use crate::codegen::JumpSite;
 use crate::codegen::opcodes::{LocalIndex, UpvalueIndex};
 use crate::codegen::funproto::UpvalueTarget;
 use crate::codegen::errors::{CompileResult, ErrorKind};
 
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub enum LocalName {
+pub(super) enum LocalName {
     // local variable names defined by AST string symbols
     Symbol(InternSymbol),
     
@@ -20,7 +22,7 @@ pub enum LocalName {
 
 
 #[derive(Debug, Clone)]
-pub struct Local {
+pub(super) struct Local {
     decl: DeclType,
     name: LocalName,
     index: LocalIndex,
@@ -28,29 +30,129 @@ pub struct Local {
 }
 
 impl Local {
-    pub fn decl(&self) -> DeclType { self.decl }
-    pub fn name(&self) -> LocalName { self.name }
-    pub fn index(&self) -> LocalIndex { self.index }
-    pub fn captured(&self) -> bool { self.captured }
+    pub(super) fn decl(&self) -> DeclType { self.decl }
+    pub(super) fn name(&self) -> LocalName { self.name }
+    pub(super) fn index(&self) -> LocalIndex { self.index }
+    pub(super) fn captured(&self) -> bool { self.captured }
 }
 
+#[derive(Clone, Copy)]
+pub(super) enum InsertLocal {
+    CreateNew,
+    HideExisting(LocalIndex),
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub(super) enum ScopeTag {
+    Block,
+    Loop,
+    Branch,
+    Function,
+}
+
+impl ScopeTag {
+    fn accepts_control_flow(&self, control_flow: ControlFlowTarget) -> bool {
+        match self {
+            Self::Block => matches!(control_flow,
+                ControlFlowTarget::Break(..)
+            ),
+            
+            Self::Loop => matches!(control_flow,
+                ControlFlowTarget::Break(..) | ControlFlowTarget::Continue(..)
+            ),
+            
+            _ => false,
+        }
+    }
+    
+    pub(super) fn is_expr_block(&self) -> bool {
+        match self {
+            Self::Block | Self:: Branch => true,
+            _ => false,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub(super) enum ControlFlowTarget {
+    Break(Option<Label>),
+    Continue(Option<Label>),
+}
+
+impl ControlFlowTarget {
+    pub(super) fn label(&self) -> Option<&Label> {
+        match self {
+            Self::Break(label) => label.as_ref(),
+            Self::Continue(label) => label.as_ref(),
+        }
+    }
+}
+
+// track break/continue jump sites
+#[derive(Debug, Default)]
+struct ControlFlowTracker {
+    label: Option<Label>,
+    continue_target: Option<usize>,
+    break_sites: Vec<JumpSite>,
+}
+
+impl ControlFlowTracker {
+    fn new(label: Option<Label>) -> Self {
+        Self {
+            label,
+            continue_target: None,
+            break_sites: Vec::new(),
+        }
+    }
+}
 
 #[derive(Debug)]
-pub struct Scope {
+pub(super) struct Scope {
+    tag: ScopeTag,
+    depth: usize,
     symbol: Option<DebugSymbol>,
     prev_index: Option<LocalIndex>,
     locals: Vec<Local>,
+    control_flow: ControlFlowTracker,
 }
 
 impl Scope {
-    pub fn locals(&self) -> &[Local] {
+    pub(super) fn tag(&self) -> ScopeTag {
+        self.tag
+    }
+    
+    pub(super) fn depth(&self) -> usize {
+        self.depth
+    }
+    
+    pub(super) fn locals(&self) -> &[Local] {
         self.locals.as_slice()
     }
     
-    pub fn debug_symbol(&self) -> Option<&DebugSymbol> {
+    pub(super) fn debug_symbol(&self) -> Option<&DebugSymbol> {
         self.symbol.as_ref()
     }
     
+    pub(super) fn continue_target(&self) -> Option<usize> {
+        self.control_flow.continue_target
+    }
+    
+    pub(super) fn set_continue(&mut self, offset: usize) {
+        self.control_flow.continue_target.replace(offset);
+    }
+    
+    pub(super) fn register_break(&mut self, break_site: JumpSite) {
+        self.control_flow.break_sites.push(break_site)
+    }
+    
+    pub(super) fn break_sites(&self) -> &[JumpSite] {
+        &self.control_flow.break_sites
+    }
+    
+    fn control_flow_mut(&mut self) -> &mut ControlFlowTracker {
+        &mut self.control_flow
+    }
+    
     fn last_index(&self) -> Option<LocalIndex> {
         self.locals.last().map_or(self.prev_index, |local| Some(local.index))
     }
@@ -79,14 +181,15 @@ impl Scope {
         Ok(self.locals.last().unwrap())
     }
     
-    fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<()> {
+    fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<InsertLocal> {
         // see if this local already exists in the current scope
         if let Some(mut local) = self.find_local_mut(&name) {
             (*local).decl = decl; // redeclare with new mutability
+            Ok(InsertLocal::HideExisting(local.index))
         } else {
             self.push_local(decl, name)?;
+            Ok(InsertLocal::CreateNew)
         }
-        Ok(())
     }
 }
 
@@ -113,13 +216,16 @@ impl NestedScopes {
         self.scopes.last_mut()
     }
     
-    fn push_scope(&mut self, symbol: Option<&DebugSymbol>) {
+    fn push_scope(&mut self, symbol: Option<&DebugSymbol>, tag: ScopeTag, label: Option<Label>) {
         let prev_index = self.scopes.last().and_then(|scope| scope.last_index());
         
         let scope = Scope {
+            tag,
+            depth: self.scopes.len(),
             prev_index,
             symbol: symbol.copied(),
             locals: Vec::new(),
+            control_flow: ControlFlowTracker::new(label),
         };
         
         self.scopes.push(scope);
@@ -130,18 +236,18 @@ impl NestedScopes {
     }
     
     /// Iterate in name resolution order
-    fn iter(&self) -> impl Iterator<Item=&Scope> {
+    fn iter_nro(&self) -> impl Iterator<Item=&Scope> {
         self.scopes.iter().rev()
     }
     
-    fn iter_mut(&mut self) -> impl Iterator<Item=&mut Scope> {
+    fn iter_nro_mut(&mut self) -> impl Iterator<Item=&mut Scope> {
         self.scopes.iter_mut().rev()
     }
 }
 
 
 #[derive(Debug, Clone)]
-pub struct Upvalue {
+pub(super) struct Upvalue {
     decl: DeclType,
     name: LocalName,
     index: UpvalueIndex,
@@ -149,23 +255,23 @@ pub struct Upvalue {
 }
 
 impl Upvalue {
-    pub fn decl(&self) -> DeclType { self.decl }
-    pub fn name(&self) -> LocalName { self.name }
-    pub fn index(&self) -> UpvalueIndex { self.index }
-    pub fn target(&self) -> UpvalueTarget { self.target }
+    pub(super) fn decl(&self) -> DeclType { self.decl }
+    pub(super) fn name(&self) -> LocalName { self.name }
+    pub(super) fn index(&self) -> UpvalueIndex { self.index }
+    pub(super) fn target(&self) -> UpvalueTarget { self.target }
 }
 
 
 #[derive(Debug)]
-pub struct ScopeFrame {
+pub(super) struct CallFrame {
     scopes: NestedScopes,
     upvalues: Vec<Upvalue>,
 }
 
-impl ScopeFrame {
+impl CallFrame {
     fn new(symbol: Option<&DebugSymbol>) -> Self {
         let mut scopes = NestedScopes::new();
-        scopes.push_scope(symbol);
+        scopes.push_scope(symbol, ScopeTag::Function, None);
         
         Self {
             scopes,
@@ -173,10 +279,10 @@ impl ScopeFrame {
         }
     }
     
-    pub fn upvalues(&self) -> &[Upvalue] { self.upvalues.as_slice() }
+    pub(super) fn upvalues(&self) -> &[Upvalue] { self.upvalues.as_slice() }
     
-    pub fn iter_locals(&self) -> impl Iterator<Item=&Local> {
-        self.scopes().iter().flat_map(|scope| scope.locals().iter())
+    pub(super) fn iter_locals(&self) -> impl Iterator<Item=&Local> {
+        self.scopes().iter_nro().flat_map(|scope| scope.locals().iter())
     }
     
     fn scopes(&self) -> &NestedScopes { &self.scopes }
@@ -223,70 +329,80 @@ impl ScopeFrame {
 
 
 #[derive(Debug)]
-pub struct ScopeTracker {
-    scopes: NestedScopes,
-    frames: Vec<ScopeFrame>,
+pub(super) struct ScopeTracker {
+    toplevel: NestedScopes,
+    frames: Vec<CallFrame>,
 }
 
 impl ScopeTracker {
-    pub fn new() -> Self {
+    pub(super) fn new() -> Self {
         Self {
-            scopes: NestedScopes::new(),
+            toplevel: NestedScopes::new(),
             frames: Vec::new(),
         }
     }
     
-    pub fn is_global_scope(&self) -> bool {
-        self.frames.is_empty() && self.scopes.is_empty()
+    pub(super) fn is_global_scope(&self) -> bool {
+        self.frames.is_empty() && self.toplevel.is_empty()
     }
     
-    pub fn is_global_frame(&self) -> bool {
+    pub(super) fn is_global_frame(&self) -> bool {
         self.frames.is_empty()
     }
     
-    pub fn push_frame(&mut self, symbol: Option<&DebugSymbol>) {
-        self.frames.push(ScopeFrame::new(symbol))
+    pub(super) fn push_frame(&mut self, symbol: Option<&DebugSymbol>) {
+        self.frames.push(CallFrame::new(symbol))
     }
     
-    pub fn pop_frame(&mut self) -> ScopeFrame {
+    pub(super) fn pop_frame(&mut self) -> CallFrame {
         self.frames.pop().expect("pop empty frames")
     }
     
-    fn current_scope(&self) -> &NestedScopes {
+    fn local_scopes(&self) -> &NestedScopes {
         self.frames.last()
-            .map_or(&self.scopes, |frame| frame.scopes())
+            .map_or(&self.toplevel, |frame| frame.scopes())
     }
     
-    fn current_scope_mut(&mut self) -> &mut NestedScopes {
+    fn local_scopes_mut(&mut self) -> &mut NestedScopes {
         self.frames.last_mut()
-            .map_or(&mut self.scopes, |frame| frame.scopes_mut())
+            .map_or(&mut self.toplevel, |frame| frame.scopes_mut())
     }
     
-    pub fn local_scope(&self) -> Option<&Scope> {
-        self.current_scope().current_scope()
+    // scopes
+    
+    pub(super) fn local_scope(&self) -> Option<&Scope> {
+        self.local_scopes().current_scope()
     }
     
-    pub fn push_scope(&mut self, symbol: Option<&DebugSymbol>) {
-        self.current_scope_mut().push_scope(symbol);
+    pub(super) fn local_scope_mut(&mut self) -> Option<&mut Scope> {
+        self.local_scopes_mut().current_scope_mut()
     }
     
-    pub fn pop_scope(&mut self) -> Scope {
-        let scope = self.current_scope_mut().pop_scope();
+    pub(super) fn push_scope(&mut self, symbol: Option<&DebugSymbol>, tag: ScopeTag, label: Option<Label>) {
+        self.local_scopes_mut().push_scope(symbol, tag, label);
+    }
+    
+    pub(super) fn pop_scope(&mut self) -> Scope {
+        let scope = self.local_scopes_mut().pop_scope();
         debug_assert!(self.frames.last().map_or(true, |frame| !frame.scopes().is_empty()), "pop last scope from call frame");
         scope
     }
     
-    pub fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<()> {
-        let scope = self.current_scope_mut().current_scope_mut().expect("insert local in global scope");
+    // local variables
+    
+    pub(super) fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<InsertLocal> {
+        let scope = self.local_scopes_mut().current_scope_mut().expect("insert local in global scope");
         scope.insert_local(decl, name)
     }
     
-    pub fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
-        self.current_scope()
-            .iter().find_map(|scope| scope.find_local(name))
+    pub(super) fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
+        self.local_scopes()
+            .iter_nro().find_map(|scope| scope.find_local(name))
     }
     
-    pub fn resolve_or_create_upval(&mut self, name: &LocalName) -> CompileResult<Option<&Upvalue>> {
+    // upvalues
+    
+    pub(super) fn resolve_or_create_upval(&mut self, name: &LocalName) -> CompileResult<Option<&Upvalue>> {
         if self.frames.is_empty() {
             return Ok(None);
         }
@@ -309,8 +425,8 @@ impl ScopeTracker {
             }
             
             // check if the local name exists in the enclosing scope
-            let enclosing = enclosing_frame.map_or(&mut self.scopes, |frame| frame.scopes_mut());
-            if let Some(local) = enclosing.iter_mut().find_map(|scope| scope.find_local_mut(name)) {
+            let enclosing = enclosing_frame.map_or(&mut self.toplevel, |frame| frame.scopes_mut());
+            if let Some(local) = enclosing.iter_nro_mut().find_map(|scope| scope.find_local_mut(name)) {
                 return Ok(Some(current_frame.create_upval_for_local(local)?.index));
             }
         }
@@ -331,10 +447,34 @@ impl ScopeTracker {
     }
     
     // helper to get a frame by index and its enclosing frame
-    fn get_frames_mut(frames: &mut [ScopeFrame], frame_idx: usize) -> (&mut ScopeFrame, Option<&mut ScopeFrame>) {
+    fn get_frames_mut(frames: &mut [CallFrame], frame_idx: usize) -> (&mut CallFrame, Option<&mut CallFrame>) {
         let (frames, _) = frames.split_at_mut(frame_idx + 1);
         let (current_frame, frames) = frames.split_last_mut().unwrap();
         let enclosing_frame = frames.split_last_mut().map(|(last, _)| last);
         (current_frame, enclosing_frame)
     }
+    
+    // control flow
+    
+    // search for a scope that matches the given control flow and label
+    pub(super) fn resolve_control_flow(&self, target: ControlFlowTarget) -> Option<&Scope> {
+        self.local_scopes()
+            .iter_nro()
+            .find_map(|scope| {
+                if scope.tag().accepts_control_flow(target) {
+                    if target.label().is_none() || target.label() == scope.control_flow.label.as_ref() {
+                        return Some(scope)
+                    }
+                }
+                None
+            })
+    }
+    
+    pub(super) fn iter_scopes(&self) -> impl Iterator<Item=&Scope> {
+        self.local_scopes().iter_nro()
+    }
+    
+    pub(super) fn iter_scopes_mut(&mut self) -> impl Iterator<Item=&mut Scope> {
+        self.local_scopes_mut().iter_nro_mut()
+    }
 }
\ No newline at end of file
