@@ -1,8 +1,7 @@
 #![cfg(test)]
 
-use crate::debug::DebugSymbol;
 use crate::lexer::{LexerBuilder, Token, TokenMeta};
-use crate::lexer::errors::{LexerError, ErrorKind};
+use crate::lexer::errors::{ErrorKind};
 use crate::lexer::tests::ErrorData;
 
 #[test]
