@@ -312,7 +312,7 @@ impl MetaObject for FloatType {
         other.as_meta().as_float().map(|other| Ok(*self <= other?))
     }
     
-    fn fmt_str(&self) -> ExecResult<StringValue> {
+    fn fmt_echo(&self) -> ExecResult<StringValue> {
         fn write_float(value: FloatType, fmt: &mut impl fmt::Write) -> fmt::Result {
             if !value.is_finite() || value.trunc() != value {
                 write!(fmt, "{}", value)
@@ -326,30 +326,7 @@ impl MetaObject for FloatType {
             Ok(StringValue::new_maybe_interned(buf))
         } else {
             let mut buf = String::new();
-            write_float(*self, &mut buf)
-                .map_err(|err| ErrorKind::Message(format!("{}", err)))?;
-            Ok(StringValue::new_maybe_interned(buf))
-        }
-    }
-    
-    fn fmt_echo(&self) -> ExecResult<StringValue> {
-        fn write_float(value: FloatType, fmt: &mut impl fmt::Write) -> fmt::Result {
-            if !value.is_finite() {
-                write!(fmt, "float(\"{}\")", value)
-            } else if value.trunc() != value {
-                write!(fmt, "{}", value)
-            } else {
-                write!(fmt, "{}.0", value)
-            }
-        }
-        
-        let mut buf = StrBuffer::<32>::new();
-        if write_float(*self, &mut buf).is_ok() {
-            Ok(StringValue::new_maybe_interned(buf))
-        } else {
-            let mut buf = String::new();
-            write_float(*self, &mut buf)
-                .map_err(|err| ErrorKind::Message(format!("{}", err)))?;
+            write_float(*self, &mut buf).map_err(ErrorKind::from)?;
             Ok(StringValue::new_maybe_interned(buf))
         }
     }
