@@ -2,13 +2,18 @@ use core::fmt;
 use core::hash::{Hash, Hasher};
 use static_assertions::const_assert_eq;
 use crate::language::{IntType, FloatType};
-use crate::runtime::types::{Tuple, UserData};
+use crate::runtime::types::{Tuple, UserData, NativeIterator, Marker};
 use crate::runtime::function::{Function, NativeFunction};
 use crate::runtime::strings::{StringValue, StringSymbol, InlineStr};
 use crate::runtime::gc::{Gc, GcTrace};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
-#[cfg(target_arch = "x86_64")]
+
+// try to use less memory on 32-bit architectures
+#[cfg(target_pointer_width = "32")]
+const_assert_eq!(core::mem::size_of::<Variant>(), 8);
+
+#[cfg(target_pointer_width = "64")]
 const_assert_eq!(core::mem::size_of::<Variant>(), 16);
 
 // Fundamental data value type
@@ -18,6 +23,8 @@ pub enum Variant {
     BoolTrue,
     BoolFalse,
     
+    Marker(Marker),
+    
     Integer(IntType),
     Float(FloatType),
     
@@ -30,10 +37,34 @@ pub enum Variant {
     Function(Gc<Function>),
     NativeFunction(Gc<NativeFunction>),
     
+    Iterator(Gc<dyn NativeIterator>),
+    
     UserData(Gc<dyn UserData>),
 }
 
+unsafe impl GcTrace for Variant {
+    #[inline]
+    fn trace(&self) {
+        match self {
+            Self::Tuple(tuple) => tuple.trace(),
+            Self::Function(fun) => fun.mark_trace(),
+            Self::NativeFunction(fun) => fun.mark_trace(),
+            Self::Iterator(iter) => iter.mark_trace(),
+            Self::UserData(data) => data.mark_trace(),
+            _ => { },
+        };
+    }
+}
+
 impl Variant {
+    pub fn marker(id: StringSymbol) -> Self {
+        Self::Marker(Marker::new(id))
+    }
+    
+    pub fn is_nil(&self) -> bool {
+        matches!(self, Variant::Nil)
+    }
+    
     pub fn as_strval(&self) -> Option<StringValue> {
         match self {
             Self::InternStr(symbol) => Some(StringValue::from(*symbol)),
@@ -118,18 +149,6 @@ impl From<NativeFunction> for Variant {
     }
 }
 
-unsafe impl GcTrace for Variant {
-    #[inline]
-    fn trace(&self) {
-        match self {
-            Self::Tuple(tuple) => tuple.trace(),
-            Self::Function(fun) => fun.mark_trace(),
-            Self::NativeFunction(fun) => fun.mark_trace(),
-            _ => { },
-        };
-    }
-}
-
 
 // Not all Variants are hashable
 impl Variant {
